@@ -4,14 +4,14 @@ Export evoked data to Nitime
 ============================
 
 """
-# Author: Denis Engemann <d.engemann@fz-juelichde>
-#         Alexandre Gramfort <gramfort@nmr.mgh.harvard.edu>
+# Author: Denis Engemann <denis.engemann@gmail.com>
+#         Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
 #
 # License: BSD (3-clause)
 
-print __doc__
+print(__doc__)
 
-from mne import fiff
+import mne
 from mne.datasets import sample
 from nitime.viz import plot_tseries
 import matplotlib.pyplot as plt
@@ -22,10 +22,10 @@ data_path = sample.data_path()
 fname = data_path + '/MEG/sample/sample_audvis-ave.fif'
 
 # Reading
-evoked = fiff.Evoked(fname, setno=0, baseline=(None, 0), proj=True)
+evoked = mne.read_evokeds(fname, condition=0, baseline=(None, 0), proj=True)
 
 # Pick channels to view
-picks = fiff.pick_types(evoked.info, meg='grad', eeg=False, exclude='bads')
+picks = mne.pick_types(evoked.info, meg='grad', eeg=False, exclude='bads')
 
 evoked_ts = evoked.to_nitime(picks=picks)
 
