@@ -6,11 +6,11 @@ Reading an STC file
 STC files contain activations on cortex ie. source
 reconstructions
 """
-# Author: Alexandre Gramfort <gramfort@nmr.mgh.harvard.edu>
+# Author: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
 #
 # License: BSD (3-clause)
 
-print __doc__
+print(__doc__)
 
 import mne
 from mne.datasets import sample
@@ -21,8 +21,8 @@ fname = data_path + '/MEG/sample/sample_audvis-meg'
 stc = mne.read_source_estimate(fname)
 
 n_vertices, n_samples = stc.data.shape
-print "stc data size: %s (nb of vertices) x %s (nb of samples)" % (
-                                                    n_vertices, n_samples)
+print("stc data size: %s (nb of vertices) x %s (nb of samples)"
+      % (n_vertices, n_samples))
 
 # View source activations
 import matplotlib.pyplot as plt
