@@ -8,9 +8,9 @@ using a cloud of digitization points for coordinate alignment
 instead of e.g. EEG-cap positions.
 
 """
-print __doc__
+print(__doc__)
 
-# Authors: Denis Engemann <d.engemann@fz-juelich.de>
+# Authors: Denis Engemann <denis.engemann@gmail.com>
 #          Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
 #
 # License: BSD (3-clause)
