@@ -6,13 +6,13 @@ Sensitivity map of SSP projections
 This example shows the sources that have a forward field
 similar to the first SSP vector correcting for ECG.
 """
-# Author: Alexandre Gramfort <gramfort@nmr.mgh.harvard.edu>
+# Author: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
 #
 # License: BSD (3-clause)
 
-print __doc__
+print(__doc__)
 
-import mne
+from mne import read_forward_solution, read_proj, sensitivity_map
 from mne.datasets import sample
 data_path = sample.data_path()
 
@@ -20,16 +20,15 @@ subjects_dir = data_path + '/subjects'
 fname = data_path + '/MEG/sample/sample_audvis-meg-eeg-oct-6-fwd.fif'
 ecg_fname = data_path + '/MEG/sample/sample_audvis_ecg_proj.fif'
 
-fwd = mne.read_forward_solution(fname, surf_ori=True)
-projs = mne.read_proj(ecg_fname)
+fwd = read_forward_solution(fname, surf_ori=True)
+projs = read_proj(ecg_fname)
 projs = projs[3:][::2]  # take only one projection per channel type
 
 # Compute sensitivity map
-ssp_ecg_map = mne.sensitivity_map(fwd, ch_type='grad', projs=projs,
-                                  mode='angle')
+ssp_ecg_map = sensitivity_map(fwd, ch_type='grad', projs=projs, mode='angle')
 
 ###############################################################################
-# Show sensitivy map
+# Show sensitivity map
 
 import matplotlib.pyplot as plt
 plt.hist(ssp_ecg_map.data.ravel())
