@@ -6,16 +6,16 @@ Find ECG artifacts
 Locate QRS component of ECG.
 
 """
-# Authors: Alexandre Gramfort <gramfort@nmr.mgh.harvard.edu>
+# Authors: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
 #
 # License: BSD (3-clause)
 
-print __doc__
+print(__doc__)
 
 import numpy as np
 import matplotlib.pyplot as plt
 import mne
-from mne import fiff
+from mne import io
 from mne.datasets import sample
 data_path = sample.data_path()
 
@@ -24,21 +24,21 @@ data_path = sample.data_path()
 raw_fname = data_path + '/MEG/sample/sample_audvis_raw.fif'
 
 # Setup for reading the raw data
-raw = fiff.Raw(raw_fname)
+raw = io.Raw(raw_fname)
 
 event_id = 999
 ecg_events, _, _ = mne.preprocessing.find_ecg_events(raw, event_id,
                                                      ch_name='MEG 1531')
 
 # Read epochs
-picks = fiff.pick_types(raw.info, meg=False, eeg=False, stim=False, eog=False,
-                        include=['MEG 1531'], exclude='bads')
+picks = mne.pick_types(raw.info, meg=False, eeg=False, stim=False, eog=False,
+                       include=['MEG 1531'], exclude='bads')
 tmin, tmax = -0.1, 0.1
 epochs = mne.Epochs(raw, ecg_events, event_id, tmin, tmax, picks=picks,
                     proj=False)
 data = epochs.get_data()
 
-print "Number of detected ECG artifacts : %d" % len(data)
+print("Number of detected ECG artifacts : %d" % len(data))
 
 ###############################################################################
 # Plot ECG artifacts
