@@ -5,11 +5,11 @@ Reading an event file
 
 Read events from a file.
 """
-# Author: Alexandre Gramfort <gramfort@nmr.mgh.harvard.edu>
+# Author: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
 #
 # License: BSD (3-clause)
 
-print __doc__
+print(__doc__)
 
 import mne
 from mne.datasets import sample
@@ -27,5 +27,5 @@ events = mne.read_events(fname, exclude=[4, 32])  # keep all but 4 and 32
 mne.write_events('events.fif', events)
 
 for ind, before, after in events[:5]:
-    print "At sample %d stim channel went from %d to %d" % (
-                                                    ind, before, after)
+    print("At sample %d stim channel went from %d to %d"
+          % (ind, before, after))
