@@ -3,4 +3,6 @@
 
 from . import sample
 from . import megsim
-from . import spm_face
\ No newline at end of file
+from . import spm_face
+from . import eegbci
+from . import somato
