@@ -1,9 +1,10 @@
-# Authors: Alexandre Gramfort <gramfort@nmr.mgh.harvard.edu>
+# Authors: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
 #          Martin Luessi <mluessi@nmr.mgh.harvard.edu>
 #          Eric Larson <larson.eric.d@gmail.com>
-#          Denis Egnemann <d.engemann@fz-juelich.de>
+#          Denis Egnemann <denis.engemann@gmail.com>
 # License: BSD Style.
 
+from ..externals.six import string_types
 import os
 import os.path as op
 import shutil
@@ -62,50 +63,70 @@ def _dataset_version(path, name):
 
 
 def _data_path(path=None, force_update=False, update_path=True,
-               download=True, name=None, verbose=None):
+               download=True, name=None, check_version=True, verbose=None):
     """Aux function
     """
     key = {'sample': 'MNE_DATASETS_SAMPLE_PATH',
-           'spm': 'MNE_DATASETS_SPM_FACE_PATH'}[name]
+           'spm': 'MNE_DATASETS_SPM_FACE_PATH',
+           'somato': 'MNE_DATASETS_SOMATO_PATH',}[name]
 
     if path is None:
         # use an intelligent guess if it's not defined
         def_path = op.realpath(op.join(op.dirname(__file__),
                                        '..', '..', 'examples'))
 
+        # backward compatibility
+        if get_config(key) is None:
+            key = 'MNE_DATA'
+
         path = get_config(key, def_path)
+
         # use the same for all datasets
-        if not os.path.exists(path):
-            path = def_path
+        if not op.exists(path) or not os.access(path, os.W_OK):
+            try:
+                os.mkdir(path)
+            except OSError:
+                try:
+                    logger.info("Checking for dataset in '~/mne_data'...")
+                    path = op.join(op.expanduser("~"), "mne_data")
+                    if not op.exists(path):
+                        logger.info("Trying to create "
+                                    "'~/mne_data' in home directory")
+                        os.mkdir(path)
+                except OSError:
+                    raise OSError("User does not have write permissions "
+                                  "at '%s', try giving the path as an argument "
+                                  "to data_path() where user has write "
+                                  "permissions, for ex:data_path"
+                                  "('/home/xyz/me2/')" % (path))
 
-    if not isinstance(path, basestring):
+    if not isinstance(path, string_types):
         raise ValueError('path must be a string or None')
-
     if name == 'sample':
         archive_name = "MNE-sample-data-processed.tar.gz"
-        url = "ftp://surfer.nmr.mgh.harvard.edu/pub/data/" + archive_name
+        url = "ftp://surfer.nmr.mgh.harvard.edu/pub/data/MNE/" + archive_name
         folder_name = "MNE-sample-data"
         folder_path = op.join(path, folder_name)
-        rm_archive = False
     elif name == 'spm':
         archive_name = 'MNE-spm-face.tar.bz2'
         url = 'ftp://surfer.nmr.mgh.harvard.edu/pub/data/MNE/' + archive_name
         folder_name = "MNE-spm-face"
         folder_path = op.join(path, folder_name)
-        rm_archive = False
+    elif name == 'somato':
+        archive_name = 'MNE-somato-data.tar.gz'
+        url = 'ftp://surfer.nmr.mgh.harvard.edu/pub/data/MNE/' + archive_name
+        folder_name = "MNE-somato-data"
+        folder_path = op.join(path, folder_name)
     else:
         raise ValueError('Sorry, the dataset "%s" is not known.' % name)
-
+    rm_archive = False
     martinos_path = '/cluster/fusion/sample_data/' + archive_name
     neurospin_path = '/neurospin/tmp/gramfort/' + archive_name
-
     if not op.exists(folder_path) and not download:
         return ''
-
     if not op.exists(folder_path) or force_update:
-        logger.info('Sample data archive %s not found at:\n%s\n'
-                    'It will be downloaded and extracted at this location.'
-                    % (archive_name, folder_path))
+        logger.info('Downloading or reinstalling '
+                    'data archive %s at location %s' % (archive_name, path))
 
         if op.exists(martinos_path):
             archive_name = martinos_path
@@ -114,27 +135,28 @@ def _data_path(path=None, force_update=False, update_path=True,
         else:
             archive_name = op.join(path, archive_name)
             rm_archive = True
+            fetch_archive = True
             if op.exists(archive_name):
-                msg = ('Archive already exists at %r. Overwrite it '
-                       '(y/[n])? ' % archive_name)
+                msg = ('Archive already exists. Overwrite it (y/[n])? ')
                 answer = raw_input(msg)
                 if answer.lower() == 'y':
                     os.remove(archive_name)
                 else:
-                    raise IOError('Archive file already exists at target '
-                                  'location %r.' % archive_name)
+                    fetch_archive = False
 
-            _fetch_file(url, archive_name, print_destination=False)
+            if fetch_archive:
+                _fetch_file(url, archive_name, print_destination=False)
 
         if op.exists(folder_path):
             shutil.rmtree(folder_path)
 
-        logger.info('Decompressiong the archive: ' + archive_name)
+        logger.info('Decompressing the archive: ' + archive_name)
         logger.info('... please be patient, this can take some time')
         for ext in ['gz', 'bz2']:  # informed guess (and the only 2 options).
             try:
                 tarfile.open(archive_name, 'r:%s' % ext).extractall(path=path)
-            except tarfile.ReadError, err:
+                break
+            except tarfile.ReadError as err:
                 logger.info('%s is %s trying "bz2"' % (archive_name, err))
 
         if rm_archive:
@@ -166,12 +188,12 @@ def _data_path(path=None, force_update=False, update_path=True,
         warn('Could not determine sample dataset version; dataset could\n'
              'be out of date. Please install the "distutils" package.')
     else:  # 0.7 < 0.7.git shoud be False, therefore strip
-        if LV(data_version) < LV(mne_version.strip('.git')):
+        if check_version and LV(data_version) < LV(mne_version.strip('.git')):
             warn('The {name} dataset (version {current}) is older than '
-                 'the mne-python (version {newest}). If the examples fail, '
-                 'you may need to update the {name} dataset by using'
-                 'force_update=True'.format(name=name, current=data_version,
-                                            newest=mne_version))
+                 'mne-python (version {newest}). If the examples fail, '
+                 'you may need to update the {name} dataset by using '
+                 'mne.datasets.{name}.data_path(force_update=True)'.format(
+                     name=name, current=data_version, newest=mne_version))
 
     return path
 
@@ -179,8 +201,7 @@ def _data_path(path=None, force_update=False, update_path=True,
 def has_dataset(name):
     """Helper for sample dataset presence"""
     endswith = {'sample': 'MNE-sample-data',
-                'spm': 'MNE-spm-face'}[name]
-    if _data_path(download=False, name=name).endswith(endswith):
-        return True
-    else:
-        return False
+                'spm': 'MNE-spm-face',
+                'somato': 'MNE-somato-data'}[name]
+    dp = _data_path(download=False, name=name, check_version=False)
+    return dp.endswith(endswith)
