@@ -1,4 +1,6 @@
 from .classifier import Scaler, FilterEstimator
 from .classifier import PSDEstimator, ConcatenateChannels
 from .mixin import TransformerMixin
-from .csp import CSP
\ No newline at end of file
+from .csp import CSP
+from .ems import compute_ems
+from .time_gen import time_generalization
