@@ -1,7 +1,7 @@
 """IO with fif files containing events
 """
 
-# Authors: Alexandre Gramfort <gramfort@nmr.mgh.harvard.edu>
+# Authors: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
 #          Matti Hamalainen <msh@nmr.mgh.harvard.edu>
 #
 # License: BSD (3-clause)
@@ -9,13 +9,13 @@
 import numpy as np
 from os.path import splitext
 
-from .fiff.constants import FIFF
-from .fiff.tree import dir_tree_find
-from .fiff.tag import read_tag
-from .fiff.open import fiff_open
-from .fiff.write import write_int, start_block, start_file, end_block, end_file
-from .fiff.pick import pick_channels
-from .utils import get_config, logger, verbose
+from .utils import check_fname, logger, verbose, _get_stim_channel
+from .io.constants import FIFF
+from .io.tree import dir_tree_find
+from .io.tag import read_tag
+from .io.open import fiff_open
+from .io.write import write_int, start_block, start_file, end_block, end_file
+from .io.pick import pick_channels
 
 
 def pick_events(events, include=None, exclude=None):
@@ -166,10 +166,11 @@ def _read_events_fif(fid, tree):
     else:
         mappings = None
 
-    if mappings is not None:
-        m_ = (m.split(':') for m in mappings.split(';'))
-        mappings = dict((k, int(v)) for k, v in m_)
-    event_list = event_list.reshape(len(event_list) / 3, 3)
+    if mappings is not None:  # deal with ':' in keys
+        m_ = [[s[::-1] for s in m[::-1].split(':', 1)]
+              for m in mappings.split(';')]
+        mappings = dict((k, int(v)) for v, k in m_)
+    event_list = event_list.reshape(len(event_list) // 3, 3)
     return event_list, mappings
 
 
@@ -202,12 +203,21 @@ def read_events(filename, include=None, exclude=None):
     -----
     This function will discard the offset line (i.e., first line with zero
     event number) if it is present in a text file.
+
+    Working with downsampled data: Events that were computed before the data
+    was decimated are no longer valid. Please recompute your events after
+    decimation.
     """
+    check_fname(filename, 'events', ('.eve', '-eve.fif', '-eve.fif.gz',
+                                     '-eve.lst', '-eve.txt'))
+
     ext = splitext(filename)[1].lower()
     if ext == '.fif' or ext == '.gz':
         fid, tree, _ = fiff_open(filename)
-        event_list, _ = _read_events_fif(fid, tree)
-        fid.close()
+        try:
+            event_list, _ = _read_events_fif(fid, tree)
+        finally:
+            fid.close()
     else:
         #  Have to read this in as float64 then convert because old style
         #  eve/lst files had a second float column that will raise errors
@@ -249,6 +259,9 @@ def write_events(filename, event_list):
     event_list : array, shape (n_events, 3)
         The list of events
     """
+    check_fname(filename, 'events', ('.eve', '-eve.fif', '-eve.fif.gz',
+                                     '-eve.lst', '-eve.txt'))
+
     ext = splitext(filename)[1].lower()
     if ext == '.fif' or ext == '.gz':
         #   Start writing...
@@ -355,8 +368,8 @@ def find_stim_steps(raw, pad_start=None, pad_stop=None, merge=0,
         raise ValueError('No stim channel found to extract event triggers.')
     data, _ = raw[picks, :]
     if np.any(data < 0):
-        logger.warn('Trigger channel contains negative values. '
-                    'Taking absolute value.')
+        logger.warning('Trigger channel contains negative values. '
+                       'Taking absolute value.')
         data = np.abs(data)  # make sure trig channel is positive
     data = data.astype(np.int)
 
@@ -376,8 +389,8 @@ def _find_events(data, first_samp, verbose=None, output='onset',
         merge = 0
 
     if np.any(data < 0):
-        logger.warn('Trigger channel contains negative values. '
-                    'Taking absolute value.')
+        logger.warning('Trigger channel contains negative values. '
+                       'Taking absolute value.')
         data = np.abs(data)  # make sure trig channel is positive
     data = data.astype(np.int)
 
@@ -432,7 +445,8 @@ def _find_events(data, first_samp, verbose=None, output='onset',
 
 @verbose
 def find_events(raw, stim_channel=None, verbose=None, output='onset',
-                consecutive='increasing', min_duration=0):
+                consecutive='increasing', min_duration=0,
+                shortest_event=2):
     """Find events from raw file
 
     Parameters
@@ -459,6 +473,9 @@ def find_events(raw, stim_channel=None, verbose=None, output='onset',
     min_duration : float
         The minimum duration of a change in the events channel required
         to consider it as an event (in seconds).
+    shortest_event : int
+        Minimum number of samples an event must last (default is 2). If the
+        duration is less than this an exception will be raised.
 
     Returns
     -------
@@ -540,6 +557,16 @@ def find_events(raw, stim_channel=None, verbose=None, output='onset',
     events = _find_events(data, raw.first_samp, verbose=verbose, output=output,
                           consecutive=consecutive, min_samples=min_samples)
 
+    # add safety check for spurious events (for ex. from neuromag syst.) by
+    # checking the number of low sample events
+    n_short_events = np.sum(np.diff(events[:, 0]) < shortest_event)
+    if n_short_events > 0:
+        raise ValueError("You have %i events shorter than the "
+                         "shortest_event. These are very unusual and you "
+                         "may want to set min_duration to a larger value e.g."
+                         " x / raw.info['sfreq']. Where x = 1 sample shorter "
+                         "than the shortest event length." % (n_short_events))
+
     return events
 
 
@@ -686,26 +713,3 @@ def concatenate_events(events, first_samps, last_samps):
         events_out = np.concatenate((events_out, e2), axis=0)
 
     return events_out
-
-
-def _get_stim_channel(stim_channel):
-    """Helper to determine the appropriate stim_channel"""
-    if stim_channel is not None:
-        if not isinstance(stim_channel, list):
-            if not isinstance(stim_channel, basestring):
-                raise ValueError('stim_channel must be a str, list, or None')
-            stim_channel = [stim_channel]
-        if not all([isinstance(s, basestring) for s in stim_channel]):
-            raise ValueError('stim_channel list must contain all strings')
-        return stim_channel
-
-    stim_channel = list()
-    ch_count = 0
-    ch = get_config('MNE_STIM_CHANNEL')
-    while(ch is not None):
-        stim_channel.append(ch)
-        ch_count += 1
-        ch = get_config('MNE_STIM_CHANNEL_%d' % ch_count)
-    if ch_count == 0:
-        stim_channel = ['STI 014']
-    return stim_channel
