@@ -1,4 +1,4 @@
-from .forward import (read_forward_solution, write_forward_solution,
+from .forward import (Forward, read_forward_solution, write_forward_solution,
                       is_fixed_orient, read_forward_meas_info,
                       write_forward_meas_info,
                       compute_orient_prior, compute_depth_prior,
@@ -10,3 +10,5 @@ from .forward import (read_forward_solution, write_forward_solution,
                       _subject_from_forward, convert_forward_solution,
                       _to_fixed_ori, prepare_bem_model)
 from ._make_forward import make_forward_solution
+from ._field_interpolation import _make_surface_mapping, make_field_map
+from . import _lead_dots  # for testing purposes
