@@ -1,8 +1,9 @@
 """Linear inverse solvers based on L2 Minimum Norm Estimates (MNE)"""
 
-from .inverse import (read_inverse_operator, apply_inverse,
+from .inverse import (InverseOperator, read_inverse_operator, apply_inverse,
                       apply_inverse_raw, make_inverse_operator,
                       apply_inverse_epochs, write_inverse_operator,
                       compute_rank_inverse)
+from .psf_ctf import point_spread_function, cross_talk_function
 from .time_frequency import (source_band_induced_power, source_induced_power,
                              compute_source_psd, compute_source_psd_epochs)
