@@ -5,7 +5,7 @@ from numpy.testing import assert_array_almost_equal
 from nose.tools import assert_true
 
 from mne.datasets import sample
-from mne import fiff, find_events, Epochs
+from mne import io, find_events, Epochs, pick_types
 from mne.label import read_label
 from mne.minimum_norm.inverse import (read_inverse_operator,
                                       apply_inverse_epochs)
@@ -32,14 +32,14 @@ def test_tfr_with_inverse_operator():
     tmin, tmax, event_id = -0.2, 0.5, 1
 
     # Setup for reading the raw data
-    raw = fiff.Raw(fname_data)
+    raw = io.Raw(fname_data)
     events = find_events(raw, stim_channel='STI 014')
     inverse_operator = read_inverse_operator(fname_inv)
 
     raw.info['bads'] += ['MEG 2443', 'EEG 053']  # bads + 2 more
 
     # picks MEG gradiometers
-    picks = fiff.pick_types(raw.info, meg=True, eeg=False, eog=True,
+    picks = pick_types(raw.info, meg=True, eeg=False, eog=True,
                             stim=False, exclude='bads')
 
     # Load condition 1
@@ -58,7 +58,7 @@ def test_tfr_with_inverse_operator():
                                      label=label)
 
     stc = stcs['alpha']
-    assert_true(len(stcs) == len(bands.keys()))
+    assert_true(len(stcs) == len(list(bands.keys())))
     assert_true(np.all(stc.data > 0))
     assert_array_almost_equal(stc.times, epochs.times)
 
@@ -85,16 +85,16 @@ def test_tfr_with_inverse_operator():
 @sample.requires_sample_data
 def test_source_psd():
     """Test source PSD computation in label"""
-    raw = fiff.Raw(fname_data)
+    raw = io.Raw(fname_data)
     inverse_operator = read_inverse_operator(fname_inv)
     label = read_label(fname_label)
     tmin, tmax = 0, 20  # seconds
     fmin, fmax = 55, 65  # Hz
-    NFFT = 2048
+    n_fft = 2048
     stc = compute_source_psd(raw, inverse_operator, lambda2=1. / 9.,
                              method="dSPM", tmin=tmin, tmax=tmax,
                              fmin=fmin, fmax=fmax, pick_ori="normal",
-                             NFFT=NFFT, label=label, overlap=0.1)
+                             n_fft=n_fft, label=label, overlap=0.1)
     assert_true(stc.times[0] >= fmin * 1e-3)
     assert_true(stc.times[-1] <= fmax * 1e-3)
     # Time max at line frequency (60 Hz in US)
@@ -106,7 +106,7 @@ def test_source_psd():
 def test_source_psd_epochs():
     """Test multi-taper source PSD computation in label from epochs"""
 
-    raw = fiff.Raw(fname_data)
+    raw = io.Raw(fname_data)
     inverse_operator = read_inverse_operator(fname_inv)
     label = read_label(fname_label)
 
@@ -115,7 +115,7 @@ def test_source_psd_epochs():
     bandwidth = 8.
     fmin, fmax = 0, 100
 
-    picks = fiff.pick_types(raw.info, meg=True, eeg=False, stim=True,
+    picks = pick_types(raw.info, meg=True, eeg=False, stim=True,
                             ecg=True, eog=True, include=['STI 014'],
                             exclude='bads')
     reject = dict(grad=4000e-13, mag=4e-12, eog=150e-6)
