@@ -1,4 +1,4 @@
-# Authors: Alexandre Gramfort <gramfort@nmr.mgh.harvard.edu>
+# Authors: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
 #          Martin Luessi <mluessi@nmr.mgh.harvard.edu>
 #
 # License: BSD (3-clause)
@@ -8,7 +8,7 @@ from warnings import warn
 import numpy as np
 from scipy import linalg, signal, fftpack
 
-from ..fiff.constants import FIFF
+from ..io.constants import FIFF
 from ..source_estimate import _make_stc
 from ..time_frequency.tfr import cwt, morlet
 from ..time_frequency.multitaper import (dpss_windows, _psd_from_mt,
@@ -19,6 +19,7 @@ from .inverse import (combine_xyz, prepare_inverse_operator, _assemble_kernel,
                       _check_ori, _subject_from_inverse)
 from ..parallel import parallel_func
 from ..utils import logger, verbose
+from ..externals import six
 
 
 @verbose
@@ -83,7 +84,7 @@ def source_band_induced_power(epochs, inverse_operator, bands, label=None,
     method = _check_method(method)
 
     frequencies = np.concatenate([np.arange(band[0], band[1] + df / 2.0, df)
-                                 for _, band in bands.iteritems()])
+                                 for _, band in six.iteritems(bands)])
 
     powers, _, vertno = _source_induced_power(epochs,
                                       inverse_operator, frequencies,
@@ -97,7 +98,7 @@ def source_band_induced_power(epochs, inverse_operator, bands, label=None,
     stcs = dict()
 
     subject = _subject_from_inverse(inverse_operator)
-    for name, band in bands.iteritems():
+    for name, band in six.iteritems(bands):
         idx = [k for k, f in enumerate(frequencies) if band[0] <= f <= band[1]]
 
         # average power in band + mean over epochs
@@ -128,7 +129,7 @@ def _compute_pow_plv(data, K, sel, Ws, source_ori, use_fft, Vh, with_plv,
     is_free_ori = False
     if (source_ori == FIFF.FIFFV_MNE_FREE_ORI and pick_ori == None):
         is_free_ori = True
-        n_sources /= 3
+        n_sources //= 3
 
     shape = (n_sources, n_freqs, n_times)
     power = np.zeros(shape, dtype=np.float)  # power
@@ -267,7 +268,7 @@ def source_induced_power(epochs, inverse_operator, frequencies, label=None,
     ----------
     epochs : instance of Epochs
         The epochs.
-    inverse_operator : instance of inverse operator
+    inverse_operator : instance of InverseOperator
         The inverse operator.
     label : Label
         Restricts the source estimates to a given label.
@@ -334,15 +335,16 @@ def source_induced_power(epochs, inverse_operator, frequencies, label=None,
 @verbose
 def compute_source_psd(raw, inverse_operator, lambda2=1. / 9., method="dSPM",
                        tmin=None, tmax=None, fmin=0., fmax=200.,
-                       NFFT=2048, overlap=0.5, pick_ori=None, label=None,
-                       nave=1, pca=True, verbose=None, pick_normal=None):
+                       n_fft=2048, overlap=0.5, pick_ori=None, label=None,
+                       nave=1, pca=True, verbose=None, pick_normal=None,
+                       NFFT=None):
     """Compute source power spectrum density (PSD)
 
     Parameters
     ----------
     raw : instance of Raw
         The raw data
-    inverse_operator : dict
+    inverse_operator : instance of InverseOperator
         The inverse operator
     lambda2: float
         The regularization parameter
@@ -358,7 +360,7 @@ def compute_source_psd(raw, inverse_operator, lambda2=1. / 9., method="dSPM",
         The lower frequency of interest
     fmax : float
         The upper frequency of interest
-    NFFT: int
+    n_fft: int
         Window size for the FFT. Should be a power of 2.
     overlap: float
         The overlap fraction between windows. Should be between 0 and 1.
@@ -383,6 +385,11 @@ def compute_source_psd(raw, inverse_operator, lambda2=1. / 9., method="dSPM",
     stc : SourceEstimate | VolSourceEstimate
         The PSD (in dB) of each of the sources.
     """
+    if NFFT is not None:
+        n_fft = NFFT
+        warnings.warn("`NFFT` is deprecated and will be removed in v0.9. "
+                      "Use `n_fft` instead")
+
     pick_ori = _check_ori(pick_ori, pick_normal)
 
     logger.info('Considering frequencies %g ... %g Hz' % (fmin, fmax))
@@ -419,19 +426,19 @@ def compute_source_psd(raw, inverse_operator, lambda2=1. / 9., method="dSPM",
         start = raw.time_as_index(tmin)[0]
     if tmax is not None:
         stop = raw.time_as_index(tmax)[0] + 1
-    NFFT = int(NFFT)
+    n_fft = int(n_fft)
     Fs = raw.info['sfreq']
-    window = signal.hanning(NFFT)
-    freqs = fftpack.fftfreq(NFFT, 1. / Fs)
+    window = signal.hanning(n_fft)
+    freqs = fftpack.fftfreq(n_fft, 1. / Fs)
     freqs_mask = (freqs >= 0) & (freqs >= fmin) & (freqs <= fmax)
     freqs = freqs[freqs_mask]
     fstep = np.mean(np.diff(freqs))
     psd = np.zeros((K.shape[0], np.sum(freqs_mask)))
     n_windows = 0
 
-    for this_start in np.arange(start, stop, int(NFFT * (1. - overlap))):
-        data, _ = raw[sel, this_start:this_start + NFFT]
-        if data.shape[1] < NFFT:
+    for this_start in np.arange(start, stop, int(n_fft * (1. - overlap))):
+        data, _ = raw[sel, this_start:this_start + n_fft]
+        if data.shape[1] < n_fft:
             logger.info("Skipping last buffer")
             break
 
@@ -568,7 +575,8 @@ def _compute_source_psd_epochs(epochs, inverse_operator, lambda2=1. / 9.,
             # compute the psd
             if adaptive:
                 out = parallel(my_psd_from_mt_adaptive(x, eigvals, freq_mask)
-                       for x in np.array_split(x_mt_src, n_jobs))
+                       for x in np.array_split(x_mt_src,
+                                               min(n_jobs, len(x_mt_src))))
                 this_psd = np.concatenate(out)
             else:
                 x_mt_src = x_mt_src[:, :, freq_mask]
@@ -606,7 +614,7 @@ def compute_source_psd_epochs(epochs, inverse_operator, lambda2=1. / 9.,
     ----------
     epochs : instance of Epochs
         The raw data.
-    inverse_operator : dict
+    inverse_operator : instance of InverseOperator
         The inverse operator.
     lambda2 : float
         The regularization parameter.
