@@ -5,13 +5,13 @@ from nose.tools import assert_true, assert_equal
 from numpy.testing import assert_array_almost_equal
 import numpy as np
 
-from ...fiff import Raw
-from ...fiff.proj import make_projector, activate_proj
+from ...io import Raw
+from ...io.proj import make_projector, activate_proj
 from ..ssp import compute_proj_ecg, compute_proj_eog
 
 warnings.simplefilter('always')  # enable b/c these tests throw warnings
 
-data_path = op.join(op.dirname(__file__), '..', '..', 'fiff', 'tests', 'data')
+data_path = op.join(op.dirname(__file__), '..', '..', 'io', 'tests', 'data')
 raw_fname = op.join(data_path, 'test_raw.fif')
 dur_use = 5.0
 eog_times = np.array([0.5, 2.3, 3.6, 14.5])
@@ -36,6 +36,7 @@ def test_compute_proj_ecg():
 
         # without setting a bad channel, this should throw a warning
         with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter('always')
             projs, events = compute_proj_ecg(raw, n_mag=2, n_grad=2, n_eeg=2,
                                              ch_name='MEG 1531', bads=[],
                                              average=average, avg_ref=True,
@@ -62,6 +63,7 @@ def test_compute_proj_eog():
 
         # This will throw a warning b/c simplefilter('always')
         with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter('always')
             projs, events = compute_proj_eog(raw, n_mag=2, n_grad=2, n_eeg=2,
                                              average=average, bads=[],
                                              avg_ref=True, no_proj=False,
