@@ -1,7 +1,8 @@
 """Time frequency analysis tools
 """
 
-from .tfr import induced_power, single_trial_power, morlet
+from .tfr import induced_power, single_trial_power, morlet, tfr_morlet
+from .tfr import AverageTFR
 from .psd import compute_raw_psd, compute_epochs_psd
 from .csd import CrossSpectralDensity, compute_epochs_csd
 from .ar import yule_walker, ar_raw, iir_filter_raw
