@@ -1,35 +1,42 @@
-# Authors : Alexandre Gramfort, gramfort@nmr.mgh.harvard.edu (2011)
-#           Denis A. Engemann <d.engemann@fz-juelich.de>
+# Authors : Alexandre Gramfort, alexandre.gramfort@telecom-paristech.fr (2011)
+#           Denis A. Engemann <denis.engemann@gmail.com>
 # License : BSD 3-clause
 
 import numpy as np
 
 from ..parallel import parallel_func
-from ..fiff.proj import make_projector_info
-from ..fiff.pick import pick_types
+from ..io.proj import make_projector_info
+from ..io.pick import pick_types
 from ..utils import logger, verbose
 
 
 @verbose
 def compute_raw_psd(raw, tmin=0, tmax=np.inf, picks=None,
-                    fmin=0, fmax=np.inf, NFFT=2048, n_jobs=1,
-                    plot=False, proj=False, verbose=None):
-    """Compute power spectral density with multi-taper
+                    fmin=0, fmax=np.inf, n_fft=2048, pad_to=None, n_overlap=0,
+                    n_jobs=1, plot=False, proj=False, NFFT=None,
+                    verbose=None):
+    """Compute power spectral density with average periodograms.
 
     Parameters
     ----------
     raw : instance of Raw
         The raw data.
-    picks : None or array of integers
+    picks : array-like of int | None
         The selection of channels to include in the computation.
         If None, take all channels.
     fmin : float
         Min frequency of interest
     fmax : float
         Max frequency of interest
-    NFFT : int
+    n_fft : int
         The length of the tapers ie. the windows. The smaller
         it is the smoother are the PSDs.
+    pad_to : int | None
+        The number of points to which the data segment is padded when
+        performing the FFT. If None, pad_to equals `NFFT`.
+    n_overlap : int
+        The number of points of overlap between blocks. The default value
+        is 0 (no overlap).
     n_jobs : int
         Number of CPUs to use in the computation.
     plot : bool
@@ -46,6 +53,10 @@ def compute_raw_psd(raw, tmin=0, tmax=np.inf, picks=None,
     freqs: array of float
         The frequencies
     """
+    if NFFT is not None:
+        n_fft = NFFT
+        warnings.warn("`NFFT` is deprecated and will be removed in v0.9. "
+                      "Use `n_fft` instead")
     start, stop = raw.time_as_index([tmin, tmax])
     if picks is not None:
         data, times = raw[picks, start:(stop + 1)]
@@ -59,19 +70,20 @@ def compute_raw_psd(raw, tmin=0, tmax=np.inf, picks=None,
         else:
             data = np.dot(proj, data)
 
-    NFFT = int(NFFT)
+    n_fft = int(n_fft)
     Fs = raw.info['sfreq']
 
-    logger.info("Effective window size : %0.3f (s)" % (NFFT / float(Fs)))
+    logger.info("Effective window size : %0.3f (s)" % (n_fft / float(Fs)))
 
     import matplotlib.pyplot as plt
     parallel, my_psd, n_jobs = parallel_func(plt.psd, n_jobs)
     fig = plt.figure()
-    out = parallel(my_psd(d, Fs=Fs, NFFT=NFFT) for d in data)
+    out = parallel(my_psd(d, Fs=Fs, NFFT=n_fft, noverlap=n_overlap,
+                          pad_to=pad_to) for d in data)
     if not plot:
         plt.close(fig)
     freqs = out[0][1]
-    psd = np.array(zip(*out)[0])
+    psd = np.array([o[0] for o in out])
 
     mask = (freqs >= fmin) & (freqs <= fmax)
     freqs = freqs[mask]
@@ -80,10 +92,11 @@ def compute_raw_psd(raw, tmin=0, tmax=np.inf, picks=None,
     return psd, freqs
 
 
-def _compute_psd(data, fmin, fmax, Fs, n_fft, psd):
+def _compute_psd(data, fmin, fmax, Fs, n_fft, psd, n_overlap, pad_to):
     """Compute the PSD"""
-    out = [psd(d, Fs=Fs, NFFT=n_fft) for d in data]
-    psd = np.array(zip(*out)[0])
+    out = [psd(d, Fs=Fs, NFFT=n_fft, noverlap=n_overlap, pad_to=pad_to)
+           for d in data]
+    psd = np.array([o[0] for o in out])
     freqs = out[0][1]
     mask = (freqs >= fmin) & (freqs <= fmax)
     freqs = freqs[mask]
@@ -92,8 +105,8 @@ def _compute_psd(data, fmin, fmax, Fs, n_fft, psd):
 
 @verbose
 def compute_epochs_psd(epochs, picks=None, fmin=0, fmax=np.inf, n_fft=256,
-                       n_jobs=1, verbose=None):
-    """Compute power spectral density with multi-taper
+                       pad_to=None, n_overlap=0, n_jobs=1, verbose=None):
+    """Compute power spectral density with with average periodograms.
 
     Parameters
     ----------
@@ -103,7 +116,7 @@ def compute_epochs_psd(epochs, picks=None, fmin=0, fmax=np.inf, n_fft=256,
         Min time instant to consider
     tmax : float
         Max time instant to consider
-    picks : None or array of integers
+    picks : array-like of int | None
         The selection of channels to include in the computation.
         If None, take all channels.
     fmin : float
@@ -113,6 +126,12 @@ def compute_epochs_psd(epochs, picks=None, fmin=0, fmax=np.inf, n_fft=256,
     n_fft : int
         The length of the tapers ie. the windows. The smaller
         it is the smoother are the PSDs.
+    pad_to : int | None
+        The number of points to which the data segment is padded when
+        performing the FFT. If None, pad_to equals `n_fft`.
+    n_overlap : int
+        The number of points of overlap between blocks. The default value
+        is 0 (no overlap).
     n_jobs : int
         Number of CPUs to use in the computation.
     verbose : bool, str, int, or None
@@ -137,9 +156,10 @@ def compute_epochs_psd(epochs, picks=None, fmin=0, fmax=np.inf, n_fft=256,
     import matplotlib.pyplot as plt
     parallel, my_psd, n_jobs = parallel_func(_compute_psd, n_jobs)
     fig = plt.figure()  # threading will induce errors otherwise
-    out = parallel(my_psd(data[picks], fmin, fmax, Fs, n_fft, plt.psd)
+    out = parallel(my_psd(data[picks], fmin, fmax, Fs, n_fft, plt.psd,
+                          n_overlap, pad_to)
                    for data in epochs)
     plt.close(fig)
-    psds, freqs = zip(*out)
-
+    psds = [o[0] for o in out]
+    freqs = [o[1] for o in out]
     return np.array(psds), freqs[0]
