@@ -1,11 +1,11 @@
 """Some utility functions"""
+from __future__ import print_function
 
-# Authors: Alexandre Gramfort <gramfort@nmr.mgh.harvard.edu>
+# Authors: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
 #
 # License: BSD (3-clause)
 
 import warnings
-import numpy as np
 import logging
 from distutils.version import LooseVersion
 import os
@@ -20,15 +20,20 @@ import tempfile
 import shutil
 from shutil import rmtree
 import atexit
-from math import log
+from math import log, ceil
 import json
-import urllib
-import urllib2
 import ftplib
-import urlparse
+import hashlib
+
+import numpy as np
 import scipy
 from scipy import linalg
 
+
+from .externals.six.moves import urllib
+from .externals.six import string_types, StringIO, BytesIO
+from .externals.decorator import decorator
+
 logger = logging.getLogger('mne')  # one selection here used across mne-python
 logger.propagate = False  # don't propagate (in case of multiple imports)
 
@@ -36,6 +41,112 @@ logger.propagate = False  # don't propagate (in case of multiple imports)
 ###############################################################################
 # RANDOM UTILITIES
 
+def _sort_keys(x):
+    """Sort and return keys of dict"""
+    keys = list(x.keys())  # note: not thread-safe
+    idx = np.argsort([str(k) for k in keys])
+    keys = [keys[ii] for ii in idx]
+    return keys
+
+
+def object_hash(x, h=None):
+    """Hash a reasonable python object
+
+    Parameters
+    ----------
+    x : object
+        Object to hash. Can be anything comprised of nested versions of:
+        {dict, list, tuple, ndarray, str, bytes, float, int, None}.
+    h : hashlib HASH object | None
+        Optional, object to add the hash to. None creates an MD5 hash.
+
+    Returns
+    -------
+    digest : int
+        The digest resulting from the hash.
+    """
+    if h is None:
+        h = hashlib.md5()
+    if isinstance(x, dict):
+        keys = _sort_keys(x)
+        for key in keys:
+            object_hash(key, h)
+            object_hash(x[key], h)
+    elif isinstance(x, (list, tuple)):
+        h.update(str(type(x)).encode('utf-8'))
+        for xx in x:
+            object_hash(xx, h)
+    elif isinstance(x, bytes):
+        # must come before "str" below
+        h.update(x)
+    elif isinstance(x, (string_types, float, int, type(None))):
+        h.update(str(type(x)).encode('utf-8'))
+        h.update(str(x).encode('utf-8'))
+    elif isinstance(x, np.ndarray):
+        x = np.asarray(x)
+        h.update(str(x.shape).encode('utf-8'))
+        h.update(str(x.dtype).encode('utf-8'))
+        h.update(x.tostring())
+    else:
+        raise RuntimeError('unsupported type: %s (%s)' % (type(x), x))
+    return int(h.hexdigest(), 16)
+
+
+def object_diff(a, b, pre=''):
+    """Compute all differences between two python variables
+
+    Parameters
+    ----------
+    a : object
+        Currently supported: dict, list, tuple, ndarray, int, str, bytes,
+        float, StringIO, BytesIO.
+    b : object
+        Must be same type as x1.
+    pre : str
+        String to prepend to each line.
+
+    Returns
+    -------
+    diffs : str
+        A string representation of the differences.
+    """
+    out = ''
+    if type(a) != type(b):
+        out += pre + ' type mismatch (%s, %s)\n' % (type(a), type(b))
+    elif isinstance(a, dict):
+        k1s = _sort_keys(a)
+        k2s = _sort_keys(b)
+        m1 = set(k2s) - set(k1s)
+        if len(m1):
+            out += pre + ' x1 missing keys %s\n' % (m1)
+        for key in k1s:
+            if key not in k2s:
+                out += pre + ' x2 missing key %s\n' % key
+            else:
+                out += object_diff(a[key], b[key], pre + 'd1[%s]' % repr(key))
+    elif isinstance(a, (list, tuple)):
+        if len(a) != len(b):
+            out += pre + ' length mismatch (%s, %s)\n' % (len(a), len(b))
+        else:
+            for xx1, xx2 in zip(a, b):
+                out += object_diff(xx1, xx2, pre='')
+    elif isinstance(a, (string_types, int, float, bytes)):
+        if a != b:
+            out += pre + ' value mismatch (%s, %s)\n' % (a, b)
+    elif a is None:
+        if b is not None:
+            out += pre + ' a is None, b is not (%s)\n' % (b)
+    elif isinstance(a, np.ndarray):
+        if not np.array_equal(a, b):
+            out += pre + ' array mismatch\n'
+    elif isinstance(a, (StringIO, BytesIO)):
+        if a.getvalue() != b.getvalue():
+            out += pre + ' StringIO mismatch\n'
+    else:
+        raise RuntimeError(pre + ': unsupported type %s (%s)' % (type(a), a))
+    return out
+
+
 def check_random_state(seed):
     """Turn seed into a np.random.RandomState instance
 
@@ -57,7 +168,7 @@ def check_random_state(seed):
 def split_list(l, n):
     """split list in n (approx) equal pieces"""
     n = int(n)
-    sz = len(l) / n
+    sz = len(l) // n
     for i in range(n - 1):
         yield l[i * sz:(i + 1) * sz]
     yield l[(n - 1) * sz:]
@@ -73,7 +184,7 @@ def create_chunks(sequence, size):
     size : int
         The chunksize to be returned
     """
-    return (sequence[p:p + size] for p in xrange(0, len(sequence), size))
+    return (sequence[p:p + size] for p in range(0, len(sequence), size))
 
 
 def sum_squared(X):
@@ -93,6 +204,25 @@ def sum_squared(X):
     return np.dot(X_flat, X_flat)
 
 
+def check_fname(fname, filetype, endings):
+    """Enforce MNE filename conventions
+
+    Parameters
+    ----------
+    fname : str
+        Name of the file.
+    filetype : str
+        Type of file. e.g., ICA, Epochs etc.
+    endings : tuple
+        Acceptable endings for the filename.
+    """
+    print_endings = ' or '.join([', '.join(endings[:-1]), endings[-1]])
+    if not fname.endswith(endings):
+        warnings.warn('This filename does not conform to mne naming convention'
+                      's. All %s files should end with '
+                      '%s' % (filetype, print_endings))
+
+
 class WrapStdOut(object):
     """Ridiculous class to work around how doctest captures stdout"""
     def __getattr__(self, name):
@@ -167,6 +297,41 @@ def estimate_rank(data, tol=1e-4, return_singular=False,
         return rank
 
 
+def _reject_data_segments(data, reject, flat, decim, info, tstep):
+    """Reject data segments using peak-to-peak amplitude
+    """
+    from .epochs import _is_good
+    from .io.pick import channel_indices_by_type
+
+    data_clean = np.empty_like(data)
+    idx_by_type = channel_indices_by_type(info)
+    step = int(ceil(tstep * info['sfreq']))
+    if decim is not None:
+        step = int(ceil(step / float(decim)))
+    this_start = 0
+    this_stop = 0
+    drop_inds = []
+    for first in range(0, data.shape[1], step):
+        last = first + step
+        data_buffer = data[:, first:last]
+        if data_buffer.shape[1] < (last - first):
+            break  # end of the time segment
+        if _is_good(data_buffer, info['ch_names'], idx_by_type, reject,
+                    flat, ignore_chs=info['bads']):
+            this_stop = this_start + data_buffer.shape[1]
+            data_clean[:, this_start:this_stop] = data_buffer
+            this_start += data_buffer.shape[1]
+        else:
+            logger.info("Artifact detected in [%d, %d]" % (first, last))
+            drop_inds.append((first, last))
+    data = data_clean[:, :this_stop]
+    if not data.any():
+        raise RuntimeError('No clean segment found. Please '
+                           'consider updating your rejection '
+                           'thresholds.')
+    return data, drop_inds
+
+
 def run_subprocess(command, *args, **kwargs):
     """Run command using subprocess.Popen
 
@@ -343,50 +508,48 @@ class deprecated(object):
         return newdoc
 
 
-def verbose(function):
-    """Decorator to allow functions to override default log level
+@decorator
+def verbose(function, *args, **kwargs):
+    """Improved verbose decorator to allow functions to override log-level
 
-    Do not call this function directly to set the global verbosity level,
-    instead use set_log_level().
+    Do not call this directly to set global verbosrity level, instead use
+    set_log_level().
 
-    Parameters (to decorated function)
-    ----------------------------------
-    verbose : bool, str, int, or None
-        The level of messages to print. If a str, it can be either DEBUG,
-        INFO, WARNING, ERROR, or CRITICAL. Note that these are for
-        convenience and are equivalent to passing in logging.DEBUG, etc.
-        For bool, True is the same as 'INFO', False is the same as 'WARNING'.
-        None defaults to using the current log level [e.g., set using
-        mne.set_log_level()].
+    Parameters
+    ----------
+    function - function
+        Function to be decorated by setting the verbosity level.
+
+    Returns
+    -------
+    dec - function
+        The decorated function
     """
     arg_names = inspect.getargspec(function).args
-    # this wrap allows decorated functions to be pickled (e.g., for parallel)
 
-    @wraps(function)
-    def dec(*args, **kwargs):
-        # Check if the first arg is "self", if it has verbose, make it default
-        if len(arg_names) > 0 and arg_names[0] == 'self':
-            default_level = getattr(args[0], 'verbose', None)
-        else:
-            default_level = None
-        verbose_level = kwargs.get('verbose', default_level)
-        if verbose_level is not None:
-            old_level = set_log_level(verbose_level, True)
-            # set it back if we get an exception
-            try:
-                ret = function(*args, **kwargs)
-            except:
-                set_log_level(old_level)
-                raise
+    if len(arg_names) > 0 and arg_names[0] == 'self':
+        default_level = getattr(args[0], 'verbose', None)
+    else:
+        default_level = None
+
+    if('verbose' in arg_names):
+        verbose_level = args[arg_names.index('verbose')]
+    else:
+        verbose_level = default_level
+
+    if verbose_level is not None:
+        old_level = set_log_level(verbose_level, True)
+        # set it back if we get an exception
+        try:
+            ret = function(*args, **kwargs)
+        except:
             set_log_level(old_level)
-            return ret
-        else:
-            return function(*args, **kwargs)
-
-    # set __wrapped__ attribute so ?? in IPython gets the right source
-    dec.__wrapped__ = function
-
-    return dec
+            raise
+        set_log_level(old_level)
+        return ret
+    else:
+        ret = function(*args, **kwargs)
+        return ret
 
 
 def has_command_line_tools():
@@ -431,6 +594,18 @@ requires_fs_or_nibabel = np.testing.dec.skipif(not has_nibabel() and
                                                'Freesurfer')
 
 
+def has_neuromag2ft():
+    """Aux function"""
+    if not 'NEUROMAG2FT_ROOT' in os.environ:
+        return False
+    else:
+        return True
+
+
+requires_neuromag2ft = np.testing.dec.skipif(not has_neuromag2ft(),
+                                             'Requires neuromag2ft')
+
+
 def requires_nibabel(vox2ras_tkr=False):
     """Aux function"""
     if vox2ras_tkr:
@@ -523,7 +698,7 @@ def requires_statsmodels(function):
     def dec(*args, **kwargs):
         skip = False
         try:
-            from tvtk.api import tvtk  # analysis:ignore
+            import statsmodels  # noqa, analysis:ignore
         except ImportError:
             skip = True
 
@@ -538,6 +713,72 @@ def requires_statsmodels(function):
     return dec
 
 
+def requires_patsy(function):
+    """
+    Decorator to skip test if patsy is not available. Patsy should be a
+    statsmodels dependency but apparently it's possible to install statsmodels
+    without it.
+    """
+    @wraps(function)
+    def dec(*args, **kwargs):
+        skip = False
+        try:
+            import patsy  # noqa, analysis:ignore
+        except ImportError:
+            skip = True
+
+        if skip is True:
+            from nose.plugins.skip import SkipTest
+            raise SkipTest('Test %s skipped, requires patsy'
+                           % function.__name__)
+        ret = function(*args, **kwargs)
+
+        return ret
+
+    return dec
+
+
+def requires_sklearn(function):
+    """Decorator to skip test if sklearn is not available"""
+    @wraps(function)
+    def dec(*args, **kwargs):
+        required_version = '0.14'
+        skip = False
+        try:
+            import sklearn
+            version = LooseVersion(sklearn.__version__)
+            if version < required_version:
+                skip = True
+        except ImportError:
+            skip = True
+
+        if skip is True:
+            from nose.plugins.skip import SkipTest
+            raise SkipTest('Test %s skipped, requires sklearn (version >= %s)'
+                           % (function.__name__, required_version))
+        ret = function(*args, **kwargs)
+
+        return ret
+
+    return dec
+
+
+def requires_good_network(function):
+    """Helper for testing"""
+
+    @wraps(function)
+    def dec(*args, **kwargs):
+        if int(os.environ.get('MNE_SKIP_NETWORK_TESTS', 0)):
+            from nose.plugins.skip import SkipTest
+            raise SkipTest('Test %s skipped, requires a good network '
+                           'connection' % function.__name__)
+        ret = function(*args, **kwargs)
+
+        return ret
+
+    return dec
+
+
 def make_skipper_dec(module, skip_str):
     """Helper to make skipping decorators"""
     skip = False
@@ -548,7 +789,6 @@ def make_skipper_dec(module, skip_str):
     return np.testing.dec.skipif(skip, skip_str)
 
 
-requires_sklearn = make_skipper_dec('sklearn', 'scikit-learn not installed')
 requires_nitime = make_skipper_dec('nitime', 'nitime not installed')
 requires_traits = make_skipper_dec('traits', 'traits not installed')
 
@@ -561,8 +801,24 @@ def _mne_fs_not_in_env():
 requires_mne_fs_in_env = np.testing.dec.skipif(_mne_fs_not_in_env)
 
 
+def _check_mayavi_version(min_version='4.3.0'):
+    """Raise a RuntimeError if the required version of mayavi is not available
+
+    Parameters
+    ----------
+    min_version : str
+        The version string. Anything that matches
+        ``'(\\d+ | [a-z]+ | \\.)'``
+    """
+    import mayavi
+    require_mayavi = LooseVersion(min_version)
+    has_mayavi = LooseVersion(mayavi.__version__)
+    if has_mayavi < require_mayavi:
+        raise RuntimeError("Need mayavi >= %s" % require_mayavi)
+
+
 def check_sklearn_version(min_version):
-    """ Check minimum sklearn version required
+    """Check minimum sklearn version required
 
     Parameters
     ----------
@@ -582,7 +838,7 @@ def check_sklearn_version(min_version):
 
 
 def check_scipy_version(min_version):
-    """ Check minimum sklearn version required
+    """Check minimum sklearn version required
 
     Parameters
     ----------
@@ -601,6 +857,25 @@ def requires_scipy_version(min_version):
                                  % min_version)
 
 
+def _check_pytables():
+    """Helper to error if Pytables is not found"""
+    try:
+        import tables as tb
+    except ImportError:
+        raise ImportError('pytables could not be imported')
+    return tb
+
+
+def requires_pytables():
+    """Helper for testing"""
+    have = True
+    try:
+        _check_pytables()
+    except ImportError:
+        have = False
+    return np.testing.dec.skipif(not have, 'Requires pytables')
+
+
 ###############################################################################
 # LOGGING
 
@@ -626,7 +901,7 @@ def set_log_level(verbose=None, return_old_level=False):
             verbose = 'INFO'
         else:
             verbose = 'WARNING'
-    if isinstance(verbose, basestring):
+    if isinstance(verbose, string_types):
         verbose = verbose.upper()
         logging_types = dict(DEBUG=logging.DEBUG, INFO=logging.INFO,
                              WARNING=logging.WARNING, ERROR=logging.ERROR,
@@ -708,6 +983,29 @@ def get_subjects_dir(subjects_dir=None, raise_error=False):
     return subjects_dir
 
 
+def _get_extra_data_path(home_dir=None):
+    """Get path to extra data (config, tables, etc.)"""
+    if home_dir is None:
+        # this has been checked on OSX64, Linux64, and Win32
+        if 'nt' == os.name.lower():
+            home_dir = os.getenv('APPDATA')
+        else:
+            # This is a more robust way of getting the user's home folder on
+            # Linux platforms (not sure about OSX, Unix or BSD) than checking
+            # the HOME environment variable. If the user is running some sort
+            # of script that isn't launched via the command line (e.g. a script
+            # launched via Upstart) then the HOME environment variable will
+            # not be set.
+            home_dir = os.path.expanduser('~')
+
+        if home_dir is None:
+            raise ValueError('mne-python config file path could '
+                             'not be determined, please report this '
+                             'error to mne-python developers')
+
+    return op.join(home_dir, '.mne')
+
+
 def get_config_path(home_dir=None):
     """Get path to standard mne-python config file
 
@@ -722,19 +1020,11 @@ def get_config_path(home_dir=None):
     config_path : str
         The path to the mne-python configuration file. On windows, this
         will be '%APPDATA%\.mne\mne-python.json'. On every other
-        system, this will be $HOME/.mne/mne-python.json.
+        system, this will be ~/.mne/mne-python.json.
     """
-    if home_dir is None:
-        # this has been checked on OSX64, Linux64, and Win32
-        home_dir = os.getenv('APPDATA' if 'nt' == os.name.lower() else 'HOME',
-                             None)
-
-    if home_dir is None:
-        raise ValueError('mne-python config file path could '
-                         'not be determined, please report this '
-                         'error to mne-python developers')
-
-    return op.join(home_dir, '.mne', 'mne-python.json')
+    val = op.join(_get_extra_data_path(home_dir=home_dir),
+                  'mne-python.json')
+    return val
 
 
 def set_cache_dir(cache_dir):
@@ -762,12 +1052,12 @@ def set_memmap_min_size(memmap_min_size):
     Parameters
     ----------
     memmap_min_size: str or None
-        Threshold on the minimum size of arrays that triggers automated memmory
+        Threshold on the minimum size of arrays that triggers automated memory
         mapping for parallel processing, e.g., '1M' for 1 megabyte.
         Use None to disable memmaping of large arrays.
     """
     if memmap_min_size is not None:
-        if not isinstance(memmap_min_size, basestring):
+        if not isinstance(memmap_min_size, string_types):
             raise ValueError('\'memmap_min_size\' has to be a string.')
         if memmap_min_size[-1] not in ['K', 'M', 'G']:
             raise ValueError('The size has to be given in kilo-, mega-, or '
@@ -780,9 +1070,11 @@ def set_memmap_min_size(memmap_min_size):
 known_config_types = [
     'MNE_BROWSE_RAW_SIZE',
     'MNE_CUDA_IGNORE_PRECISION',
+    'MNE_DATA',
     'MNE_DATASETS_MEGSIM_PATH',
     'MNE_DATASETS_SAMPLE_PATH',
     'MNE_DATASETS_SPM_FACE_PATH',
+    'MNE_DATASETS_EEGBCI_PATH',
     'MNE_LOGGING_LEVEL',
     'MNE_USE_CUDA',
     'SUBJECTS_DIR',
@@ -798,14 +1090,15 @@ known_config_wildcards = [
     ]
 
 
-def get_config(key, default=None, raise_error=False, home_dir=None):
+def get_config(key=None, default=None, raise_error=False, home_dir=None):
     """Read mne(-python) preference from env, then mne-python config
 
     Parameters
     ----------
-    key : str
+    key : None | str
         The preference key to look for. The os evironment is searched first,
         then the mne-python config file is parsed.
+        If None, all the config parameters present in the path are returned.
     default : str | None
         Value to return if the key is not found.
     raise_error : bool
@@ -817,15 +1110,15 @@ def get_config(key, default=None, raise_error=False, home_dir=None):
 
     Returns
     -------
-    value : str | None
+    value : dict | str | None
         The preference key value.
     """
 
-    if not isinstance(key, basestring):
+    if key is not None and not isinstance(key, string_types):
         raise ValueError('key must be a string')
 
     # first, check to see if key is in env
-    if key in os.environ:
+    if key is not None and key in os.environ:
         return os.environ[key]
 
     # second, look for it in mne-python config file
@@ -836,16 +1129,19 @@ def get_config(key, default=None, raise_error=False, home_dir=None):
     else:
         with open(config_path, 'r') as fid:
             config = json.load(fid)
-        key_found = True if key in config else False
+            if key is None:
+                return config
+        key_found = key in config
         val = config.get(key, default)
 
     if not key_found and raise_error is True:
         meth_1 = 'os.environ["%s"] = VALUE' % key
         meth_2 = 'mne.utils.set_config("%s", VALUE)' % key
         raise KeyError('Key "%s" not found in environment or in the '
-                       'mne-python config file:\n%s\nTry either:\n'
-                       '    %s\nfor a temporary solution, or:\n'
-                       '    %s\nfor a permanent one. You can also '
+                       'mne-python config file: %s '
+                       'Try either:'
+                       ' %s for a temporary solution, or:'
+                       ' %s for a permanent one. You can also '
                        'set the environment variable before '
                        'running python.'
                        % (key, config_path, meth_1, meth_2))
@@ -866,11 +1162,11 @@ def set_config(key, value, home_dir=None):
         The folder that contains the .mne config folder.
         If None, it is found automatically.
     """
-    if not isinstance(key, basestring):
+    if not isinstance(key, string_types):
         raise ValueError('key must be a string')
     # While JSON allow non-string types, we allow users to override config
     # settings using env, which are strings, so we enforce that here
-    if not isinstance(value, basestring) and value is not None:
+    if not isinstance(value, string_types) and value is not None:
         raise ValueError('value must be a string or None')
     if not key in known_config_types and not \
             any(k in key for k in known_config_wildcards):
@@ -890,8 +1186,9 @@ def set_config(key, value, home_dir=None):
     else:
         config[key] = value
 
-    # Write all values
-    directory = op.split(config_path)[0]
+    # Write all values. This may fail if the default directory is not
+    # writeable.
+    directory = op.dirname(config_path)
     if not op.isdir(directory):
         os.mkdir(directory)
     with open(config_path, 'w') as fid:
@@ -1011,7 +1308,7 @@ class ProgressBar(object):
         self.update(self.cur_value, mesg)
 
 
-class _HTTPResumeURLOpener(urllib.FancyURLopener):
+class _HTTPResumeURLOpener(urllib.request.FancyURLopener):
     """Create sub-class in order to overide error 206.
 
     This error means a partial file is being sent, which is ok in this case.
@@ -1032,7 +1329,7 @@ def _chunk_read(response, local_file, chunk_size=65536, initial_size=0):
 
     Parameters
     ----------
-    response: urllib.addinfourl
+    response: urllib.response.addinfourl
         Response to the download request in order to get file size.
     local_file: file
         Hard disk file where data should be written.
@@ -1047,7 +1344,7 @@ def _chunk_read(response, local_file, chunk_size=65536, initial_size=0):
     bytes_so_far = initial_size
     # Returns only amount left to download when resuming, not the size of the
     # entire file
-    total_size = int(response.info().getheader('Content-Length').strip())
+    total_size = int(response.headers['Content-Length'].strip())
     total_size += initial_size
 
     progress = ProgressBar(total_size, initial_value=bytes_so_far,
@@ -1066,14 +1363,17 @@ def _chunk_read_ftp_resume(url, temp_file_name, local_file):
     # Adapted from: https://pypi.python.org/pypi/fileDownloader.py
     # but with changes
 
-    parsed_url = urlparse.urlparse(url)
+    parsed_url = urllib.parse.urlparse(url)
     file_name = os.path.basename(parsed_url.path)
     server_path = parsed_url.path.replace(file_name, "")
-    unquoted_server_path = urllib.unquote(server_path)
+    unquoted_server_path = urllib.parse.unquote(server_path)
     local_file_size = os.path.getsize(temp_file_name)
 
     data = ftplib.FTP()
-    data.connect(parsed_url.hostname, parsed_url.port)
+    if parsed_url.port is not None:
+        data.connect(parsed_url.hostname, parsed_url.port)
+    else:
+        data.connect(parsed_url.hostname)
     data.login()
     if len(server_path) > 1:
         data.cwd(unquoted_server_path)
@@ -1087,6 +1387,7 @@ def _chunk_read_ftp_resume(url, temp_file_name, local_file):
     # chunk and will write it to file and update the progress bar
     chunk_write = lambda chunk: _chunk_write(chunk, local_file, progress)
     data.retrbinary(down_cmd, chunk_write)
+    data.close()
 
 
 def _chunk_write(chunk, local_file, progress):
@@ -1118,14 +1419,17 @@ def _fetch_file(url, file_name, print_destination=True, resume=True):
     initial_size = 0
     try:
         # Checking file size and displaying it alongside the download url
-        u = urllib2.urlopen(url)
-        file_size = int(u.info().getheaders("Content-Length")[0])
-        print 'Downloading data from %s (%s)' % (url, sizeof_fmt(file_size))
+        u = urllib.request.urlopen(url)
+        try:
+            file_size = int(u.headers['Content-Length'].strip())
+        finally:
+            del u
+        print('Downloading data from %s (%s)' % (url, sizeof_fmt(file_size)))
         # Downloading data
         if resume and os.path.exists(temp_file_name):
             local_file = open(temp_file_name, "ab")
             # Resuming HTTP and FTP downloads requires different procedures
-            scheme = urlparse.urlparse(url).scheme
+            scheme = urllib.parse.urlparse(url).scheme
             if scheme == 'http':
                 url_opener = _HTTPResumeURLOpener()
                 local_file_size = os.path.getsize(temp_file_name)
@@ -1133,35 +1437,35 @@ def _fetch_file(url, file_name, print_destination=True, resume=True):
                 url_opener.addheader("Range", "bytes=%s-" % (local_file_size))
                 try:
                     data = url_opener.open(url)
-                except urllib2.HTTPError:
+                except urllib.request.HTTPError:
                     # There is a problem that may be due to resuming, some
                     # servers may not support the "Range" header. Switch back
                     # to complete download method
-                    print 'Resuming download failed. Attempting to restart '\
-                          'downloading the entire file.'
+                    print('Resuming download failed. Attempting to restart '
+                          'downloading the entire file.')
                     _fetch_file(url, resume=False)
-                _chunk_read(data, local_file, initial_size=local_file_size)
+                else:
+                    _chunk_read(data, local_file, initial_size=local_file_size)
+                    del data  # should auto-close
             else:
                 _chunk_read_ftp_resume(url, temp_file_name, local_file)
         else:
             local_file = open(temp_file_name, "wb")
-            data = urllib2.urlopen(url)
-            _chunk_read(data, local_file, initial_size=initial_size)
+            data = urllib.request.urlopen(url)
+            try:
+                _chunk_read(data, local_file, initial_size=initial_size)
+            finally:
+                del data  # should auto-close
         # temp file must be closed prior to the move
         if not local_file.closed:
             local_file.close()
         shutil.move(temp_file_name, file_name)
         if print_destination is True:
             stdout.write('File saved as %s.\n' % file_name)
-    except urllib2.HTTPError, e:
-        print 'Error while fetching file %s.' \
-            ' Dataset fetching aborted.' % url
-        print "HTTP Error:", e, url
-        raise
-    except urllib2.URLError, e:
-        print 'Error while fetching file %s.' \
-            ' Dataset fetching aborted.' % url
-        print "URL Error:", e, url
+    except Exception as e:
+        logger.error('Error while fetching file %s.'
+                     ' Dataset fetching aborted.' % url)
+        logger.error("Error: %s", e)
         raise
     finally:
         if local_file is not None:
@@ -1171,13 +1475,14 @@ def _fetch_file(url, file_name, print_destination=True, resume=True):
 
 def sizeof_fmt(num):
     """Turn number of bytes into human-readable str"""
-    unit_list = zip(['bytes', 'kB', 'MB', 'GB', 'TB', 'PB'],
-                    [0, 0, 1, 2, 2, 2])
+    units = ['bytes', 'kB', 'MB', 'GB', 'TB', 'PB']
+    decimals = [0, 0, 1, 2, 2, 2]
     """Human friendly file size"""
     if num > 1:
-        exponent = min(int(log(num, 1024)), len(unit_list) - 1)
+        exponent = min(int(log(num, 1024)), len(units) - 1)
         quotient = float(num) / 1024 ** exponent
-        unit, num_decimals = unit_list[exponent]
+        unit = units[exponent]
+        num_decimals = decimals[exponent]
         format_string = '{0:.%sf} {1}' % (num_decimals)
         return format_string.format(quotient, unit)
     if num == 0:
@@ -1188,17 +1493,41 @@ def sizeof_fmt(num):
 
 def _url_to_local_path(url, path):
     """Mirror a url path in a local destination (keeping folder structure)"""
-    destination = urlparse.urlparse(url).path
+    destination = urllib.parse.urlparse(url).path
     # First char should be '/', and it needs to be discarded
     if len(destination) < 2 or destination[0] != '/':
         raise ValueError('Invalid URL')
-    destination = os.path.join(path, urllib2.url2pathname(destination)[1:])
+    destination = os.path.join(path,
+                               urllib.request.url2pathname(destination)[1:])
     return destination
 
 
+def _get_stim_channel(stim_channel):
+    """Helper to determine the appropriate stim_channel"""
+    if stim_channel is not None:
+        if not isinstance(stim_channel, list):
+            if not isinstance(stim_channel, string_types):
+                raise ValueError('stim_channel must be a str, list, or None')
+            stim_channel = [stim_channel]
+        if not all([isinstance(s, string_types) for s in stim_channel]):
+            raise ValueError('stim_channel list must contain all strings')
+        return stim_channel
+
+    stim_channel = list()
+    ch_count = 0
+    ch = get_config('MNE_STIM_CHANNEL')
+    while(ch is not None):
+        stim_channel.append(ch)
+        ch_count += 1
+        ch = get_config('MNE_STIM_CHANNEL_%d' % ch_count)
+    if ch_count == 0:
+        stim_channel = ['STI 014']
+    return stim_channel
+
+
 def _check_fname(fname, overwrite):
     """Helper to check for file existence"""
-    if not isinstance(fname, basestring):
+    if not isinstance(fname, string_types):
         raise TypeError('file name is not a string')
     if op.isfile(fname):
         if not overwrite:
@@ -1211,12 +1540,12 @@ def _check_fname(fname, overwrite):
 def _check_subject(class_subject, input_subject, raise_error=True):
     """Helper to get subject name from class"""
     if input_subject is not None:
-        if not isinstance(input_subject, basestring):
+        if not isinstance(input_subject, string_types):
             raise ValueError('subject input must be a string')
         else:
             return input_subject
     elif class_subject is not None:
-        if not isinstance(class_subject, basestring):
+        if not isinstance(class_subject, string_types):
             raise ValueError('Neither subject input nor class subject '
                              'attribute was a string')
         else:
@@ -1259,7 +1588,7 @@ def _clean_names(names, remove_whitespace=False, before_dash=True):
 
     Usage
     -----
-    # for new VectorView (only inside layout) 
+    # for new VectorView (only inside layout)
     ch_names = _clean_names(epochs.ch_names, remove_whitespace=True)
 
     # for CTF
@@ -1275,3 +1604,30 @@ def _clean_names(names, remove_whitespace=False, before_dash=True):
         cleaned.append(name)
 
     return cleaned
+
+
+def clean_warning_registry():
+    """Safe way to reset warnings """
+    warnings.resetwarnings()
+    reg = "__warningregistry__"
+    bad_names = ['MovedModule']  # this is in six.py, and causes bad things
+    for mod in list(sys.modules.values()):
+        if mod.__class__.__name__ not in bad_names and hasattr(mod, reg):
+            getattr(mod, reg).clear()
+
+
+def _check_type_picks(picks):
+    """helper to guarantee type integrity of picks"""
+    err_msg = 'picks must be None, a list or an array of integers'
+    if picks is None:
+        pass
+    elif isinstance(picks, list):
+        if not all([isinstance(i, int) for i in picks]):
+            raise ValueError(err_msg)
+        picks = np.array(picks)
+    elif isinstance(picks, np.ndarray):
+        if not picks.dtype.kind == 'i':
+            raise ValueError(err_msg)
+    else:
+        raise ValueError(err_msg)
+    return picks
